package com.chelvc.cloud.maintain.controller;

import com.chelvc.cloud.maintain.copier.MerchantCopier;
import com.chelvc.cloud.maintain.vo.SimpleMerchantVO;
import com.chelvc.cloud.vehicle.api.constant.GoodsStatus;
import com.chelvc.cloud.vehicle.api.dto.BalanceDetailDTO;
import com.chelvc.cloud.vehicle.api.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.api.dto.MerchantDetailDTO;
import com.chelvc.cloud.vehicle.api.dto.MerchantRankDTO;
import com.chelvc.cloud.vehicle.api.param.*;
import com.chelvc.cloud.vehicle.api.service.BalanceDetailService;
import com.chelvc.cloud.vehicle.api.service.FavoriteService;
import com.chelvc.cloud.vehicle.api.service.GoodsService;
import com.chelvc.cloud.vehicle.api.service.MerchantService;
import com.chelvc.framework.base.annotation.ResponseWrapping;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Pagination;
import org.apache.dubbo.config.annotation.DubboReference;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 商家接口
 *
 * @author Woody
 * @date 2023/7/20
 */
@Validated
@RestController
@ResponseWrapping
public class MerchantController {
    @DubboReference
    private MerchantService merchantService;

    @DubboReference
    private FavoriteService favoriteService;

    @DubboReference
    private BalanceDetailService balanceDetailService;
    @DubboReference
    private GoodsService goodsService;

    /**
     * 获取商家列表
     *
     * @param param 查询参数
     * @return 商家信息列表
     */
    @GetMapping("/merchantSort")
    public List<SimpleMerchantVO> listSimpleMerchants(@Valid MerchantQueryParam param) {
        return MerchantCopier.INSTANCE.copying(this.merchantService.listSimpleMerchants(param));
    }

    /**
     * 根据地区获取商家排名
     *
     * @param param 查询参数
     * @return 商家信息列表
     */
    @GetMapping("/merchants/rank")
    public List<MerchantRankDTO> listRankMerchants(@Valid MerchantRankParam param) {
        return this.merchantService.listRankMerchants(param, SessionContextHolder.getId());
    }

    /**
     * 获取商家详情
     *
     * @param id 商家ID
     * @param param 查询参数
     * @return 商家详情
     */
    @GetMapping("/merchant/{id}")
    public MerchantDetailDTO getMerchantDetail(@PathVariable("id") @Min(value = 1, message = "商家ID不能小于1") Long id,
                                               @Valid LocationQueryParam param) {
        return this.merchantService.getMerchantDetail(id, param);
    }

    /**
     * 收藏商家信息
     *
     * @param id 商家ID
     * @return 收藏ID
     */
    @PostMapping("/merchant/{id}/favorite")
    public Long addMerchantFavorite(@PathVariable("id") @Min(value = 1, message = "商家ID不能小于1") Long id) {
        return this.favoriteService.addMerchantFavorite(id,"MERCHANT");
    }

    /**
     * 查询商家余额明细
     *
     * @param param 查询条件
     * @param pageNum
     * @param pageSize
     * @return 收藏ID
     */
    @GetMapping("/merchant/balance/detail")
    public Pagination<BalanceDetailDTO> queryBalanceDetail(@Validated QueryMerchantBalanceParam param, Integer pageNum, Integer pageSize) {
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }
        return this.balanceDetailService.queryByUserId(param, pageNum, pageSize, SessionContextHolder.getId(), 2);
    }


    /**
     * 我的购物车列表
     *
     */
    @GetMapping("/mineCartList")
    public Pagination<MerchantDTO> mineCartList(@Valid OmsCartItemPagingParam param) {
        return this.merchantService.getUserMineCartPaging(param);
    }


    /**
     * 上架商品
     *
     * @param goodsIds    商品ID集合
     */
    @PutMapping("/merchant/list-goods/{goodsIds}")
    public void listGoods(@PathVariable("goodsIds") @NotEmpty(message = "商品ID不能为空") List<Long> goodsIds) {
        this.goodsService.updateGoodsStatus(goodsIds, GoodsStatus.ONLINE);
    }

    /**
     * 下架商品
     *
     * @param goodsIds    商品ID集合
     */
    @PutMapping("/merchant/delist-goods/{goodsIds}")
    public void delistGoods(@PathVariable("goodsIds") @NotEmpty(message = "商品ID不能为空") List<Long> goodsIds) {
        this.goodsService.updateGoodsStatus(goodsIds, GoodsStatus.OFFLINE);
    }


    /**
     * 修改商家营业状态
     *
     * @param id    商家主键
     * @param businessStatus 修改参数  营业状态 0-营业中，1-休息中
     */
    @PutMapping("/merchant/updateBusinessStatus/{id}")
    public void updateMerchant(@PathVariable("id") @Min(value = 1, message = "商家主键不能小于1") Long id,
                               String businessStatus) {
        this.merchantService.updateMerchantBusinessStatus(id,businessStatus);
    }

}
